mod modem;

use esp_idf_hal::prelude::*;
use esp_idf_hal::peripherals::Peripherals;
use esp_idf_hal::serial;

fn start_loop<UART: serial::Uart>(_m: &mut modem::Modem<UART>) -> () {
    println!("Starting rx/tx loop (not implemented yet) ...");
}

struct Ap<'a> {
    apn: &'a str,
    username: &'a str,
    password: &'a str,
}

const A1: Ap = Ap {
    apn: "internet",
    username: "internet",
    password: "internet",
};

fn main() {
    esp_idf_sys::link_patches();

    let dp = Peripherals::take().unwrap();

    let modem_pwrkey = dp.pins.gpio4.into_output().unwrap();
    let modem_rst = dp.pins.gpio5.into_output().unwrap();
    let modem_power = dp.pins.gpio23.into_output().unwrap();

    modem::init(modem_pwrkey, modem_rst, modem_power).unwrap();

    let rx_pin = dp.pins.gpio26;
    let tx_pin = dp.pins.gpio27;

    let serial_pins = serial::Pins {
        tx: tx_pin,
        rx: rx_pin,
        cts: None,
        rts: None,
    };

    println!("Setting up serial ...");
    let serial: serial::Serial<serial::UART1, _, _> = serial::Serial::new(
        dp.uart1,
        serial_pins,
        serial::config::Config::default().baudrate(Hertz(9600)),
    ).expect("Failed to create serial ... bailing");
    println!("Serial done ...");

    println!("Writing to serial TX ...");

    let (tx, rx) = serial.split();

    let mut modem = modem::Modem::new(tx, rx);
    let _ = modem.connect_to_gprs_ap(A1.apn, A1.username, A1.password).unwrap();
    let _ = modem.test_modem().unwrap();
}
